import asyncio
from typing import ClassVar, List, Optional

from google.protobuf.json_format import MessageToDict
from grpc.aio import insecure_channel
from pydantic import (
    UUID4,
    BaseModel,
    StringConstraints,
    validate_call,
)
from typing_extensions import Annotated

from .api_constants import (
    SUCCESS_CODE,
    ControllerStubs,
    InvokerUnits,
    RequestMessages,
    RPCMethodNames,
)

IranianPhoneNumber = Annotated[
    str,
    StringConstraints(
        strip_whitespace=True, pattern=r"^(?:\+98|098|98|0098|)0?9\d{9}$"
    ),
]


class ValidationError(Exception):
    pass


class ResponseCodeError(Exception):
    pass


class RPCClient(BaseModel):
    """
    Configure the host and port of the search server.

    Parameters:
        host (str): The host address of the search server.
        port (int): The port number of the search server.
    """

    host: str = "127.0.0.1"
    port: str | int = "50051"
    api_key: str

    _instances: ClassVar[dict] = {}

    def __new__(cls, *args, **kwargs):
        # Ensure that each subclass has its own singleton instance
        if cls not in cls._instances:
            cls._instances[cls] = super().__new__(cls)
        return cls._instances[cls]

    async def perform_operation(self, rpc_method_name, request_message=None):
        """Perform the operation.

        Parameters:
            rpc_method_name (str): The name of the RPC method (e.g., `Comment`).
            request_message (gRPC message): The request message instance.
        """
        async with insecure_channel(f"{self.host}:{self.port}") as channel:
            self.controller_stub.channel = channel
            self.controller_stub.ready()
            RPCMethod = getattr(self.controller_stub, rpc_method_name, None)

            if RPCMethod:
                metadata = [("api-key", self.api_key)]
                response = await RPCMethod(request_message, metadata=metadata)

                # Clear the request message for future use if is not None
                if request_message is not None:
                    request_message.Clear()

                # Raise error if response is not successful
                if response.code != SUCCESS_CODE:
                    raise ResponseCodeError(response.code)

                return MessageToDict(response, preserving_proto_field_name=True)
            else:
                raise ValidationError("Invalid RPC method name.")

    async def perform_bulk_operation(self, rpc_method_name, request_messages: List):
        """Perform bulk operation (multiple requests at once).

        Parameters:
            rpc_method_name (str): The name of the RPC method (e.g., `fetch_one`).
            request_messages (List[gRPC message]): A list of request message instances.
        """
        async with insecure_channel(f"{self.host}:{self.port}") as channel:
            self.controller_stub.channel = channel
            self.controller_stub.ready()
            RPCMethod = getattr(self.controller_stub, rpc_method_name, None)

            if RPCMethod:
                metadata = [("api-key", self.api_key)]

                # Send all requests concurrently
                responses = await asyncio.gather(
                    *[
                        RPCMethod(request, metadata=metadata)
                        for request in request_messages
                    ]
                )

                # Process responses
                response_dicts = []
                for response in responses:
                    if response.code != SUCCESS_CODE:
                        raise ResponseCodeError(response.code)
                    response_dicts.append(
                        MessageToDict(response, preserving_proto_field_name=True)
                    )

                return response_dicts
            else:
                raise ValidationError("Invalid RPC method name.")


class RPCBase(RPCClient):
    """Base class for RPC operations, extending RPCClient for handling RPC calls."""

    comment_request: ClassVar = RequestMessages.COMMENT

    @validate_call
    async def fetch_one(
        self, identifier: Optional[str | UUID4] = None, slug: Optional[str] = None
    ):
        """Fetch a single record from a database model."""
        if identifier:
            self.fetch_one_request.identifier = identifier  # Assign attributes
        if slug:
            self.fetch_one_request.slug = slug  # Assign attributes

        return await self.perform_operation(
            RPCMethodNames.FETCH_ONE, self.fetch_one_request
        )

    @validate_call
    async def fetch_all(self, order_by: str, page_size: int, page_number: int):
        """Fetch all records from a database model."""
        self.fetch_all_request.order_by = order_by
        self.fetch_all_request.page_size = page_size
        self.fetch_all_request.page_number = page_number

        return await self.perform_operation(
            RPCMethodNames.FETCH_ALL, self.fetch_all_request
        )

    @validate_call
    async def fetch_all_for_sitemap(
        self, order_by: str, page_size: int, page_number: int
    ):
        """Fetch all records from a database model for sitemap use."""
        self.fetch_all_request.order_by = order_by
        self.fetch_all_request.page_size = page_size
        self.fetch_all_request.page_number = page_number
        self.fetch_all_request.invoker_unit = InvokerUnits.GATEWAY_SITEMAP

        return await self.perform_operation(
            RPCMethodNames.FETCH_ALL, self.fetch_all_request
        )

    @validate_call
    async def fetch_comments(
        self, identifier: Optional[str | UUID4] = None, slug: Optional[str] = None
    ):
        """Fetch a all `comments` of an item object."""
        # Assign attributes
        if identifier:
            RequestMessages.FETCH_COURSE_COMMENTS.identifier = identifier
        if slug:
            RequestMessages.FETCH_COURSE_COMMENTS.slug = slug

        if not isinstance(self, Package):
            return await self.perform_operation(
                RPCMethodNames.FETCH_COMMENTS, RequestMessages.FETCH_COURSE_COMMENTS
            )
        raise ValidationError("package has no comments.")

    @validate_call
    async def comment(
        self,
        name: str,
        content: str,
        owner_identifier: Optional[str | UUID4] = None,
        owner_slug: Optional[str] = None,
        user_identifier: Optional[str | UUID4] = None,
    ):
        """Comment on a course/lesson."""
        # Assign attributes
        self.comment_request.name = name
        self.comment_request.content = content
        # Since these are optional don't set if None
        if owner_identifier:
            self.comment_request.owner_identifier = owner_identifier
        if owner_slug:
            self.comment_request.owner_slug = owner_slug
        if user_identifier:
            self.comment_request.user_identifier = user_identifier

        if not isinstance(self, Package):
            return await self.perform_operation(
                RPCMethodNames.COMMENT,
                self.comment_request,
            )
        raise ValidationError("Cannot comment on a package.")


class CourseBase(RPCBase):
    """`fetch_rates()`, `fetch_comments()`."""

    fetch_one_request: ClassVar = RequestMessages.FETCH_COURSE
    fetch_all_request: ClassVar = RequestMessages.FETCH_ALL_COURSE

    @validate_call
    async def fetch_rates(
        self, identifier: Optional[str | UUID4] = None, slug: Optional[str] = None
    ):
        """Fetch a all `rates` of a course object."""
        # Assign attributes
        if identifier:
            RequestMessages.FETCH_COURSE_RATES.identifier = identifier
        if slug:
            RequestMessages.FETCH_COURSE_RATES.slug = slug

        return await self.perform_operation(
            RPCMethodNames.FETCH_RATES, RequestMessages.FETCH_COURSE_RATES
        )


class Course(CourseBase):
    """`fetch_one()`, `featch_all()`, `comment()`, `like()`, `unlike()`, `fetch_chapters()`, `fetch_rates()`, `fetch_comments()`."""

    controller_stub: ClassVar = ControllerStubs.COURSE

    @validate_call
    async def fetch_normals(self, order_by: str, page_size: int, page_number: int):
        """Fetch all normal courses."""
        self.fetch_all_request.order_by = order_by
        self.fetch_all_request.page_size = page_size
        self.fetch_all_request.page_number = page_number
        self.fetch_all_request.item_type = "course"

        return await self.perform_operation(
            RPCMethodNames.FETCH_ALL, self.fetch_all_request
        )

    @validate_call
    async def fetch_roadmaps(self, order_by: str, page_size: int, page_number: int):
        """Fetch all roadmap courses."""
        self.fetch_all_request.order_by = order_by
        self.fetch_all_request.page_size = page_size
        self.fetch_all_request.page_number = page_number
        self.fetch_all_request.item_type = "roadmap"

        return await self.perform_operation(
            RPCMethodNames.FETCH_ALL, self.fetch_all_request
        )

    @validate_call
    async def like(
        self,
        course_identifier: Optional[str | UUID4] = None,
        course_slug: Optional[str] = None,
    ):
        # Since these are optional don't set if None
        if course_identifier:
            RequestMessages.LIKE.owner_identifier = course_identifier
        if course_slug:
            RequestMessages.LIKE.owner_slug = course_slug

        return await self.perform_operation(RPCMethodNames.LIKE, RequestMessages.LIKE)

    @validate_call
    async def unlike(
        self,
        course_identifier: Optional[str | UUID4] = None,
        course_slug: Optional[str] = None,
    ):
        # Since these are optional don't set if None
        if course_identifier:
            RequestMessages.UNLIKE.owner_identifier = course_identifier
        if course_slug:
            RequestMessages.UNLIKE.owner_slug = course_slug

        return await self.perform_operation(
            RPCMethodNames.UNLIKE, RequestMessages.UNLIKE
        )

    @validate_call
    async def fetch_chapters(
        self,
        course_identifier: Optional[str | UUID4] = None,
        course_slug: Optional[str] = None,
    ):
        # Since these are optional don't set if None
        if course_identifier:
            RequestMessages.FETCH_COURSE_CHAPTERS.identifier = course_identifier
        if course_slug:
            RequestMessages.FETCH_COURSE_CHAPTERS.slug = course_slug

        return await self.perform_operation(
            RPCMethodNames.FETCH_COURSE_CHAPTERS, RequestMessages.FETCH_COURSE_CHAPTERS
        )

    @validate_call
    async def rate(
        self,
        user_identifier: str | UUID4,
        rate: float,
        course_identifier: Optional[str | UUID4] = None,
        course_slug: Optional[str] = None,
    ):
        # Assign attributes
        RequestMessages.RATE.user_identifier = user_identifier
        RequestMessages.RATE.rate = rate
        # Since these are optional don't set if None
        if course_identifier:
            RequestMessages.RATE.owner_identifier = course_identifier
        if course_slug:
            RequestMessages.RATE.owner_slug = course_slug

        return await self.perform_operation(RPCMethodNames.RATE, RequestMessages.RATE)


class MasterCourse(Course):
    """`fetch_one()`, `featch_all()`, `comment()`, `like()`, `unlike()`, `fetch_rates()`, `fetch_comments()`."""

    controller_stub: ClassVar = ControllerStubs.MASTER_COURSE


class Lesson(RPCBase):
    """`fetch_one()`, `featch_all()`, `comment()`."""

    controller_stub: ClassVar = ControllerStubs.LESSON
    fetch_one_request: ClassVar = RequestMessages.FETCH_LESSON
    fetch_all_request: ClassVar = RequestMessages.FETCH_ALL_LESSON


class Package(RPCBase):
    """`fetch_one()`, `featch_all()`."""

    controller_stub: ClassVar = ControllerStubs.PAKCAGE
    fetch_one_request: ClassVar = RequestMessages.FETCH_PACKAGE
    fetch_all_request: ClassVar = RequestMessages.FETCH_ALL_PACKAGE


class Category(RPCClient):
    """`hierarchical_fetch_all()`."""

    controller_stub: ClassVar = ControllerStubs.CATEGORY

    async def hierarchical_fetch_all(self):
        """Fetch all records in a hierarchical structure."""
        return await self.perform_operation(RPCMethodNames.HIERARCHICAL_FETCH_ALL)

    @validate_call
    async def fetch_one(
        self, identifier: Optional[str | UUID4] = None, slug: Optional[str] = None
    ):
        """Fetch a single record from a database model."""
        if identifier:
            RequestMessages.FETCH_CATEGORY.identifier = identifier
        if slug:
            RequestMessages.FETCH_CATEGORY.slug = slug

        return await self.perform_operation(
            RPCMethodNames.FETCH_ONE, RequestMessages.FETCH_CATEGORY
        )


class TopBanner(RPCClient):
    """`hierarchical_fetch_all()`."""

    controller_stub: ClassVar = ControllerStubs.TOP_BANNER

    @validate_call
    async def fetch_current(self):
        """Fetch the current top banner object."""
        return await self.perform_operation(RPCMethodNames.FETCH_CURRENT)


class General(RPCClient):
    """`fetch_one()`, `fetch_bulk()`."""

    controller_stub: ClassVar = ControllerStubs.GENERAL

    @validate_call
    async def fetch_one_for_asset(self, identifier: Optional[str | UUID4]):
        """Fetch item by it's `identifeir` regardless of its type."""
        # Assign attributes
        RequestMessages.FETCH_ITEM.identifier = identifier
        RequestMessages.FETCH_ITEM.invoker_unit = InvokerUnits.ACCOUNT_ASSET

        return await self.perform_operation(
            RPCMethodNames.FETCH_ONE, RequestMessages.FETCH_ITEM
        )

    @validate_call
    async def fetch_one_for_basket(self, identifier: Optional[str | UUID4]):
        """Fetch item by it's `identifeir` regardless of its type."""
        # Assign attributes
        RequestMessages.FETCH_ITEM.identifier = identifier
        RequestMessages.FETCH_ITEM.invoker_unit = InvokerUnits.CORE_BASKET

        return await self.perform_operation(
            RPCMethodNames.FETCH_ONE, RequestMessages.FETCH_ITEM
        )

    @validate_call
    async def fetch_one_for_asset_manager(self, identifier: Optional[str | UUID4]):
        """Fetch item by it's `identifeir` regardless of its type."""
        # Assign attributes
        RequestMessages.FETCH_ITEM.identifier = identifier
        RequestMessages.FETCH_ITEM.invoker_unit = InvokerUnits.CORE_ASSET_MANAGEMENT

        return await self.perform_operation(
            RPCMethodNames.FETCH_ONE, RequestMessages.FETCH_ITEM
        )

    @validate_call
    async def fetch_bulk_for_asset(self, identifiers: List[Optional[str | UUID4]]):
        """Fetch multiple items by their identifiers."""
        # Prepare a list of request messages for all identifiers
        request_messages = [
            RequestMessages.FETCH_BULK_ITEM(
                identifier=identifier, invoker_unit=InvokerUnits.ACCOUNT_ASSET
            )
            for identifier in identifiers
        ]

        return await self.perform_bulk_operation(
            RPCMethodNames.FETCH_ONE, request_messages
        )

    @validate_call
    async def fetch_bulk_for_basket(self, identifiers: List[Optional[str | UUID4]]):
        """Fetch multiple items by their identifiers."""
        # Prepare a list of request messages for all identifiers
        request_messages = [
            RequestMessages.FETCH_BULK_ITEM(
                identifier=identifier, invoker_unit=InvokerUnits.CORE_BASKET
            )
            for identifier in identifiers
        ]

        return await self.perform_bulk_operation(
            RPCMethodNames.FETCH_ONE, request_messages
        )

    @validate_call
    async def fetch_bulk_for_invoice(self, identifiers: List[Optional[str | UUID4]]):
        """Fetch multiple items by their identifiers for use in invoice inquiry."""
        # Prepare a list of request messages for all identifiers
        request_messages = [
            RequestMessages.FETCH_BULK_ITEM(
                identifier=identifier, invoker_unit=InvokerUnits.INVOICE_INQUIRY
            )
            for identifier in identifiers
        ]

        return await self.perform_bulk_operation(
            RPCMethodNames.FETCH_ONE, request_messages
        )


class Consultant(RPCClient):
    """`create()`."""

    controller_stub: ClassVar = ControllerStubs.CONSULTANT

    @validate_call
    async def create(
        self,
        phone_number: IranianPhoneNumber,
        full_name: str,
        item_type: Optional[str] = None,
        item_identifier: Optional[str | UUID4] = None,
    ):
        """create a new Consultant request."""
        # Assign attributes
        RequestMessages.CREATE_CONSULTANT_REQUEST.phone_number = phone_number
        RequestMessages.CREATE_CONSULTANT_REQUEST.full_name = full_name
        if item_type:
            RequestMessages.CREATE_CONSULTANT_REQUEST.item_type = item_type
        if item_identifier:
            RequestMessages.CREATE_CONSULTANT_REQUEST.item_identifier = item_identifier

        return await self.perform_operation(
            RPCMethodNames.CREATE, RequestMessages.CREATE_CONSULTANT_REQUEST
        )



# pylint: skip-file
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: course_lesson/grpc/course_lesson.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n&course_lesson/grpc/course_lesson.proto\x12\x14\x63onfig.course_lesson\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1cgoogle/protobuf/struct.proto"\xa7\x01\n\x11\x41\x64\x64\x43ommentRequest\x12\x1d\n\x10owner_identifier\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\t\x12\x17\n\x0fuser_identifier\x18\x04 \x01(\t\x12\x17\n\nowner_slug\x18\x05 \x01(\tH\x01\x88\x01\x01\x42\x13\n\x11_owner_identifierB\r\n\x0b_owner_slug"Z\n\x12\x41\x64\x64\x43ommentResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.Struct"X\n\x10\x43\x61tegoryResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.Struct";\n\x17\x43\x61tegoryRetrieveRequest\x12\x12\n\nidentifier\x18\x01 \x01(\t\x12\x0c\n\x04slug\x18\x02 \x01(\t"\x94\x01\n\x11\x43onsultantRequest\x12\x11\n\tfull_name\x18\x02 \x01(\t\x12\x14\n\x0cphone_number\x18\x03 \x01(\t\x12\x16\n\titem_type\x18\x04 \x01(\tH\x00\x88\x01\x01\x12\x1c\n\x0fitem_identifier\x18\x05 \x01(\tH\x01\x88\x01\x01\x42\x0c\n\n_item_typeB\x12\n\x10_item_identifier"Z\n\x12\x43onsultantResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.Struct"\x9b\x01\n\x11\x43ourseListRequest\x12\x15\n\x08order_by\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x13\n\x0bpage_number\x18\x02 \x01(\x05\x12\x11\n\tpage_size\x18\x03 \x01(\x05\x12\x16\n\titem_type\x18\x04 \x01(\tH\x01\x88\x01\x01\x12\x14\n\x0cinvoker_unit\x18\x05 \x01(\tB\x0b\n\t_order_byB\x0c\n\n_item_type"V\n\x0e\x43ourseResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.Struct"A\n\x1d\x43ourseRetrieveCommentsRequest\x12\x12\n\nidentifier\x18\x01 \x01(\t\x12\x0c\n\x04slug\x18\x02 \x01(\t">\n\x1a\x43ourseRetrieveRatesRequest\x12\x12\n\nidentifier\x18\x01 \x01(\t\x12\x0c\n\x04slug\x18\x02 \x01(\t"9\n\x15\x43ourseRetrieveRequest\x12\x12\n\nidentifier\x18\x01 \x01(\t\x12\x0c\n\x04slug\x18\x02 \x01(\t"W\n\x0fGeneralResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.Struct"B\n\x16GeneralRetrieveRequest\x12\x12\n\nidentifier\x18\x01 \x01(\t\x12\x14\n\x0cinvoker_unit\x18\x02 \x01(\t"u\n\x11LessonListRequest\x12\x15\n\x08order_by\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x13\n\x0bpage_number\x18\x02 \x01(\x05\x12\x11\n\tpage_size\x18\x03 \x01(\x05\x12\x14\n\x0cinvoker_unit\x18\x04 \x01(\tB\x0b\n\t_order_by"V\n\x0eLessonResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.Struct"A\n\x1dLessonRetrieveCommentsRequest\x12\x12\n\nidentifier\x18\x01 \x01(\t\x12\x0c\n\x04slug\x18\x02 \x01(\t"9\n\x15LessonRetrieveRequest\x12\x12\n\nidentifier\x18\x01 \x01(\t\x12\x0c\n\x04slug\x18\x02 \x01(\t"*\n\rMasterRequest\x12\x19\n\x11master_identifier\x18\x01 \x01(\t"V\n\x0eMasterResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.Struct"v\n\x12PackageListRequest\x12\x15\n\x08order_by\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x13\n\x0bpage_number\x18\x02 \x01(\x05\x12\x11\n\tpage_size\x18\x03 \x01(\x05\x12\x14\n\x0cinvoker_unit\x18\x04 \x01(\tB\x0b\n\t_order_by"W\n\x0fPackageResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.Struct":\n\x16PackageRetrieveRequest\x12\x12\n\nidentifier\x18\x01 \x01(\t\x12\x0c\n\x04slug\x18\x02 \x01(\t"\x90\x01\n\x0bRateRequest\x12\x1d\n\x10owner_identifier\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x17\n\x0fuser_identifier\x18\x02 \x01(\t\x12\x0c\n\x04rate\x18\x03 \x01(\x02\x12\x17\n\nowner_slug\x18\x04 \x01(\tH\x01\x88\x01\x01\x42\x13\n\x11_owner_identifierB\r\n\x0b_owner_slug"T\n\x0cRateResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.Struct"Y\n\x11TopBannerResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.Struct"p\n\x12TriggerLikeRequest\x12\x1d\n\x10owner_identifier\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x17\n\nowner_slug\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x13\n\x11_owner_identifierB\r\n\x0b_owner_slug"[\n\x13TriggerLikeResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\x0c\n\x04\x63ode\x18\x02 \x01(\t\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.Struct2\xcf\x01\n\x12\x43\x61tegoryController\x12T\n\x10HierarchicalList\x12\x16.google.protobuf.Empty\x1a&.config.course_lesson.CategoryResponse"\x00\x12\x63\n\x08Retrieve\x12-.config.course_lesson.CategoryRetrieveRequest\x1a&.config.course_lesson.CategoryResponse"\x00\x32u\n\x14\x43onsultantController\x12]\n\x06\x43reate\x12\'.config.course_lesson.ConsultantRequest\x1a(.config.course_lesson.ConsultantResponse"\x00\x32\xfc\x06\n\x10\x43ourseController\x12^\n\x07\x43omment\x12\'.config.course_lesson.AddCommentRequest\x1a(.config.course_lesson.AddCommentResponse"\x00\x12\x61\n\x12\x46\x65tchMasterCourses\x12#.config.course_lesson.MasterRequest\x1a$.config.course_lesson.MasterResponse"\x00\x12]\n\x04Like\x12(.config.course_lesson.TriggerLikeRequest\x1a).config.course_lesson.TriggerLikeResponse"\x00\x12W\n\x04List\x12\'.config.course_lesson.CourseListRequest\x1a$.config.course_lesson.CourseResponse"\x00\x12O\n\x04Rate\x12!.config.course_lesson.RateRequest\x1a".config.course_lesson.RateResponse"\x00\x12_\n\x08Retrieve\x12+.config.course_lesson.CourseRetrieveRequest\x1a$.config.course_lesson.CourseResponse"\x00\x12o\n\x10RetrieveComments\x12\x33.config.course_lesson.CourseRetrieveCommentsRequest\x1a$.config.course_lesson.CourseResponse"\x00\x12i\n\rRetrieveRates\x12\x30.config.course_lesson.CourseRetrieveRatesRequest\x1a$.config.course_lesson.CourseResponse"\x00\x12_\n\x06UnLike\x12(.config.course_lesson.TriggerLikeRequest\x1a).config.course_lesson.TriggerLikeResponse"\x00\x32v\n\x11GeneralController\x12\x61\n\x08Retrieve\x12,.config.course_lesson.GeneralRetrieveRequest\x1a%.config.course_lesson.GeneralResponse"\x00\x32\x9d\x03\n\x10LessonController\x12^\n\x07\x43omment\x12\'.config.course_lesson.AddCommentRequest\x1a(.config.course_lesson.AddCommentResponse"\x00\x12W\n\x04List\x12\'.config.course_lesson.LessonListRequest\x1a$.config.course_lesson.LessonResponse"\x00\x12_\n\x08Retrieve\x12+.config.course_lesson.LessonRetrieveRequest\x1a$.config.course_lesson.LessonResponse"\x00\x12o\n\x10RetrieveComments\x12\x33.config.course_lesson.LessonRetrieveCommentsRequest\x1a$.config.course_lesson.LessonResponse"\x00\x32\x82\x07\n\x16MasterCourseController\x12^\n\x07\x43omment\x12\'.config.course_lesson.AddCommentRequest\x1a(.config.course_lesson.AddCommentResponse"\x00\x12\x61\n\x12\x46\x65tchMasterCourses\x12#.config.course_lesson.MasterRequest\x1a$.config.course_lesson.MasterResponse"\x00\x12]\n\x04Like\x12(.config.course_lesson.TriggerLikeRequest\x1a).config.course_lesson.TriggerLikeResponse"\x00\x12W\n\x04List\x12\'.config.course_lesson.CourseListRequest\x1a$.config.course_lesson.CourseResponse"\x00\x12O\n\x04Rate\x12!.config.course_lesson.RateRequest\x1a".config.course_lesson.RateResponse"\x00\x12_\n\x08Retrieve\x12+.config.course_lesson.CourseRetrieveRequest\x1a$.config.course_lesson.CourseResponse"\x00\x12o\n\x10RetrieveComments\x12\x33.config.course_lesson.CourseRetrieveCommentsRequest\x1a$.config.course_lesson.CourseResponse"\x00\x12i\n\rRetrieveRates\x12\x30.config.course_lesson.CourseRetrieveRatesRequest\x1a$.config.course_lesson.CourseResponse"\x00\x12_\n\x06UnLike\x12(.config.course_lesson.TriggerLikeRequest\x1a).config.course_lesson.TriggerLikeResponse"\x00\x32\xd1\x01\n\x11PackageController\x12Y\n\x04List\x12(.config.course_lesson.PackageListRequest\x1a%.config.course_lesson.PackageResponse"\x00\x12\x61\n\x08Retrieve\x12,.config.course_lesson.PackageRetrieveRequest\x1a%.config.course_lesson.PackageResponse"\x00\x32k\n\x13TopBannerController\x12T\n\x0fRetrieveCurrent\x12\x16.google.protobuf.Empty\x1a\'.config.course_lesson.TopBannerResponse"\x00\x62\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(
    DESCRIPTOR, "course_lesson.grpc.course_lesson_pb2", _globals
)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_ADDCOMMENTREQUEST"]._serialized_start = 124
    _globals["_ADDCOMMENTREQUEST"]._serialized_end = 291
    _globals["_ADDCOMMENTRESPONSE"]._serialized_start = 293
    _globals["_ADDCOMMENTRESPONSE"]._serialized_end = 383
    _globals["_CATEGORYRESPONSE"]._serialized_start = 385
    _globals["_CATEGORYRESPONSE"]._serialized_end = 473
    _globals["_CATEGORYRETRIEVEREQUEST"]._serialized_start = 475
    _globals["_CATEGORYRETRIEVEREQUEST"]._serialized_end = 534
    _globals["_CONSULTANTREQUEST"]._serialized_start = 537
    _globals["_CONSULTANTREQUEST"]._serialized_end = 685
    _globals["_CONSULTANTRESPONSE"]._serialized_start = 687
    _globals["_CONSULTANTRESPONSE"]._serialized_end = 777
    _globals["_COURSELISTREQUEST"]._serialized_start = 780
    _globals["_COURSELISTREQUEST"]._serialized_end = 935
    _globals["_COURSERESPONSE"]._serialized_start = 937
    _globals["_COURSERESPONSE"]._serialized_end = 1023
    _globals["_COURSERETRIEVECOMMENTSREQUEST"]._serialized_start = 1025
    _globals["_COURSERETRIEVECOMMENTSREQUEST"]._serialized_end = 1090
    _globals["_COURSERETRIEVERATESREQUEST"]._serialized_start = 1092
    _globals["_COURSERETRIEVERATESREQUEST"]._serialized_end = 1154
    _globals["_COURSERETRIEVEREQUEST"]._serialized_start = 1156
    _globals["_COURSERETRIEVEREQUEST"]._serialized_end = 1213
    _globals["_GENERALRESPONSE"]._serialized_start = 1215
    _globals["_GENERALRESPONSE"]._serialized_end = 1302
    _globals["_GENERALRETRIEVEREQUEST"]._serialized_start = 1304
    _globals["_GENERALRETRIEVEREQUEST"]._serialized_end = 1370
    _globals["_LESSONLISTREQUEST"]._serialized_start = 1372
    _globals["_LESSONLISTREQUEST"]._serialized_end = 1489
    _globals["_LESSONRESPONSE"]._serialized_start = 1491
    _globals["_LESSONRESPONSE"]._serialized_end = 1577
    _globals["_LESSONRETRIEVECOMMENTSREQUEST"]._serialized_start = 1579
    _globals["_LESSONRETRIEVECOMMENTSREQUEST"]._serialized_end = 1644
    _globals["_LESSONRETRIEVEREQUEST"]._serialized_start = 1646
    _globals["_LESSONRETRIEVEREQUEST"]._serialized_end = 1703
    _globals["_MASTERREQUEST"]._serialized_start = 1705
    _globals["_MASTERREQUEST"]._serialized_end = 1747
    _globals["_MASTERRESPONSE"]._serialized_start = 1749
    _globals["_MASTERRESPONSE"]._serialized_end = 1835
    _globals["_PACKAGELISTREQUEST"]._serialized_start = 1837
    _globals["_PACKAGELISTREQUEST"]._serialized_end = 1955
    _globals["_PACKAGERESPONSE"]._serialized_start = 1957
    _globals["_PACKAGERESPONSE"]._serialized_end = 2044
    _globals["_PACKAGERETRIEVEREQUEST"]._serialized_start = 2046
    _globals["_PACKAGERETRIEVEREQUEST"]._serialized_end = 2104
    _globals["_RATEREQUEST"]._serialized_start = 2107
    _globals["_RATEREQUEST"]._serialized_end = 2251
    _globals["_RATERESPONSE"]._serialized_start = 2253
    _globals["_RATERESPONSE"]._serialized_end = 2337
    _globals["_TOPBANNERRESPONSE"]._serialized_start = 2339
    _globals["_TOPBANNERRESPONSE"]._serialized_end = 2428
    _globals["_TRIGGERLIKEREQUEST"]._serialized_start = 2430
    _globals["_TRIGGERLIKEREQUEST"]._serialized_end = 2542
    _globals["_TRIGGERLIKERESPONSE"]._serialized_start = 2544
    _globals["_TRIGGERLIKERESPONSE"]._serialized_end = 2635
    _globals["_CATEGORYCONTROLLER"]._serialized_start = 2638
    _globals["_CATEGORYCONTROLLER"]._serialized_end = 2845
    _globals["_CONSULTANTCONTROLLER"]._serialized_start = 2847
    _globals["_CONSULTANTCONTROLLER"]._serialized_end = 2964
    _globals["_COURSECONTROLLER"]._serialized_start = 2967
    _globals["_COURSECONTROLLER"]._serialized_end = 3859
    _globals["_GENERALCONTROLLER"]._serialized_start = 3861
    _globals["_GENERALCONTROLLER"]._serialized_end = 3979
    _globals["_LESSONCONTROLLER"]._serialized_start = 3982
    _globals["_LESSONCONTROLLER"]._serialized_end = 4395
    _globals["_MASTERCOURSECONTROLLER"]._serialized_start = 4398
    _globals["_MASTERCOURSECONTROLLER"]._serialized_end = 5296
    _globals["_PACKAGECONTROLLER"]._serialized_start = 5299
    _globals["_PACKAGECONTROLLER"]._serialized_end = 5508
    _globals["_TOPBANNERCONTROLLER"]._serialized_start = 5510
    _globals["_TOPBANNERCONTROLLER"]._serialized_end = 5617
# @@protoc_insertion_point(module_scope)



from .grpc_lib.course_lesson_pb2 import (
    AddCommentRequest,
    CategoryRetrieveRequest,
    ConsultantRequest,
    CourseListRequest,
    CourseRetrieveCommentsRequest,
    CourseRetrieveRatesRequest,
    CourseRetrieveRequest,
    GeneralRetrieveRequest,
    LessonListRequest,
    LessonRetrieveRequest,
    PackageListRequest,
    PackageRetrieveRequest,
    RateRequest,
    TriggerLikeRequest,
)
from .grpc_lib.course_lesson_pb2_grpc import (
    CategoryControllerStub,
    ConsultantControllerStub,
    CourseControllerStub,
    GeneralControllerStub,
    LessonControllerStub,
    MasterCourseControllerStub,
    PackageControllerStub,
    TopBannerControllerStub,
)

SUCCESS_CODE = "0"


class RequestMessages:
    CREATE_CONSULTANT_REQUEST = ConsultantRequest()
    COMMENT = AddCommentRequest()
    RATE = RateRequest()
    LIKE = TriggerLikeRequest()
    UNLIKE = TriggerLikeRequest()
    FETCH_ALL_COURSE = CourseListRequest()
    FETCH_COURSE = CourseRetrieveRequest()
    FETCH_ALL_LESSON = LessonListRequest()
    FETCH_LESSON = LessonRetrieveRequest()
    FETCH_ALL_PACKAGE = PackageListRequest()
    FETCH_PACKAGE = PackageRetrieveRequest()
    FETCH_ITEM = GeneralRetrieveRequest()
    FETCH_BULK_ITEM = GeneralRetrieveRequest
    FETCH_COURSE_RATES = CourseRetrieveRatesRequest()
    FETCH_COURSE_COMMENTS = CourseRetrieveCommentsRequest()
    FETCH_CATEGORY = CategoryRetrieveRequest()


class RPCMethodNames:
    COMMENT = "Comment"
    CREATE = "Create"
    RATE = "Rate"
    LIKE = "Like"
    UNLIKE = "UnLike"
    FETCH_ONE = "Retrieve"
    FETCH_CURRENT = "RetrieveCurrent"
    FETCH_COMMENTS = "RetrieveComments"
    FETCH_RATES = "RetrieveRates"
    FETCH_ALL = "List"
    HIERARCHICAL_FETCH_ALL = "HierarchicalList"


class ControllerStubs:
    COURSE = CourseControllerStub()
    CONSULTANT = ConsultantControllerStub()
    MASTER_COURSE = MasterCourseControllerStub()
    LESSON = LessonControllerStub()
    PAKCAGE = PackageControllerStub()
    CATEGORY = CategoryControllerStub()
    TOP_BANNER = TopBannerControllerStub()
    GENERAL = GeneralControllerStub()


class InvokerUnits:
    ACCOUNT_ASSET = "account_asset"
    CORE_BASKET = "core_basket"
    CORE_ASSET_MANAGEMENT = "core_asset_management"
    INVOICE_INQUIRY = "invoice_inquiry"
    GATEWAY_SITEMAP = "gateway_sitemap"



from setuptools import setup

with open("README.md", "r") as f:
    long_description = f.read()


setup(
    name="dmcl",
    version="2.15.0",
    author="Ellnamin",
    author_email="ellnamin.business@gmail.com",
    description="Course Lesson Service for Darsman Academy",
    long_description=long_description,
    packages=["dmcl", "dmcl/grpc_lib"],
    install_requires=[
        "grpcio==1.62.1",
        "grpcio-tools==1.62.1",
        "protobuf==4.25.3",
        "setuptools==69.1.1",
        "annotated-types==0.6.0",
        "pydantic==2.7.0",
        "pydantic_core==2.18.1",
    ],
    setup_requires=["wheel", "setuptools"],
)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""

import grpc
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2

from . import course_lesson_pb2 as course__lesson_dot_grpc_dot_course__lesson__pb2


class CategoryControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel=None):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.channel = channel
        self.ready()

    def ready(self):
        if self.channel is not None:
            self.HierarchicalList = self.channel.unary_unary(
                "/config.course_lesson.CategoryController/HierarchicalList",
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CategoryResponse.FromString,
            )
            self.Retrieve = self.channel.unary_unary(
                "/config.course_lesson.CategoryController/Retrieve",
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CategoryRetrieveRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CategoryResponse.FromString,
            )


class CategoryControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def HierarchicalList(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def Retrieve(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_CategoryControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "HierarchicalList": grpc.unary_unary_rpc_method_handler(
            servicer.HierarchicalList,
            request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CategoryResponse.SerializeToString,
        ),
        "Retrieve": grpc.unary_unary_rpc_method_handler(
            servicer.Retrieve,
            request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CategoryRetrieveRequest.FromString,
            response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CategoryResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "config.course_lesson.CategoryController", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class CategoryController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def HierarchicalList(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.course_lesson.CategoryController/HierarchicalList",
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.CategoryResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def Retrieve(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.course_lesson.CategoryController/Retrieve",
            course__lesson_dot_grpc_dot_course__lesson__pb2.CategoryRetrieveRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.CategoryResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )


class ConsultantControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel=None):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.channel = channel
        self.ready()

    def ready(self):
        if self.channel is not None:
            self.Create = self.channel.unary_unary(
                "/config.course_lesson.ConsultantController/Create",
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.ConsultantRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.ConsultantResponse.FromString,
            )


class ConsultantControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Create(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_ConsultantControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "Create": grpc.unary_unary_rpc_method_handler(
            servicer.Create,
            request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.ConsultantRequest.FromString,
            response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.ConsultantResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "config.course_lesson.ConsultantController", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class ConsultantController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Create(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.course_lesson.ConsultantController/Create",
            course__lesson_dot_grpc_dot_course__lesson__pb2.ConsultantRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.ConsultantResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )


class CourseControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel=None):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.channel = channel
        self.ready()

    def ready(self):
        if self.channel is not None:
            self.Comment = self.channel.unary_unary(
                "/config.course_lesson.CourseController/Comment",
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentResponse.FromString,
            )
            self.FetchMasterCourses = self.channel.unary_unary(
                "/config.course_lesson.CourseController/FetchMasterCourses",
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.MasterRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.MasterResponse.FromString,
            )
            self.Like = self.channel.unary_unary(
                "/config.course_lesson.CourseController/Like",
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeResponse.FromString,
            )
            self.List = self.channel.unary_unary(
                "/config.course_lesson.CourseController/List",
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseListRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
            )
            self.Rate = self.channel.unary_unary(
                "/config.course_lesson.CourseController/Rate",
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.RateRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.RateResponse.FromString,
            )
            self.Retrieve = self.channel.unary_unary(
                "/config.course_lesson.CourseController/Retrieve",
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
            )
            self.UnLike = self.channel.unary_unary(
                "/config.course_lesson.CourseController/UnLike",
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeResponse.FromString,
            )
            self.RetrieveComments = self.channel.unary_unary(
                "/config.course_lesson.CourseController/RetrieveComments",
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveCommentsRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
            )
            self.RetrieveRates = self.channel.unary_unary(
                "/config.course_lesson.CourseController/RetrieveRates",
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveRatesRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
            )


class CourseControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Comment(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def FetchMasterCourses(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def Like(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def List(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def Rate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def Retrieve(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def RetrieveComments(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def RetrieveRates(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def UnLike(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_CourseControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "Comment": grpc.unary_unary_rpc_method_handler(
            servicer.Comment,
            request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentRequest.FromString,
            response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentResponse.SerializeToString,
        ),
        "FetchMasterCourses": grpc.unary_unary_rpc_method_handler(
            servicer.FetchMasterCourses,
            request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.MasterRequest.FromString,
            response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.MasterResponse.SerializeToString,
        ),
        "Like": grpc.unary_unary_rpc_method_handler(
            servicer.Like,
            request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeRequest.FromString,
            response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeResponse.SerializeToString,
        ),
        "List": grpc.unary_unary_rpc_method_handler(
            servicer.List,
            request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseListRequest.FromString,
            response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.SerializeToString,
        ),
        "Rate": grpc.unary_unary_rpc_method_handler(
            servicer.Rate,
            request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.RateRequest.FromString,
            response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.RateResponse.SerializeToString,
        ),
        "Retrieve": grpc.unary_unary_rpc_method_handler(
            servicer.Retrieve,
            request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveRequest.FromString,
            response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.SerializeToString,
        ),
        "RetrieveComments": grpc.unary_unary_rpc_method_handler(
            servicer.RetrieveComments,
            request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveCommentsRequest.FromString,
            response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.SerializeToString,
        ),
        "RetrieveRates": grpc.unary_unary_rpc_method_handler(
            servicer.RetrieveRates,
            request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveRatesRequest.FromString,
            response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.SerializeToString,
        ),
        "UnLike": grpc.unary_unary_rpc_method_handler(
            servicer.UnLike,
            request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeRequest.FromString,
            response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "config.course_lesson.CourseController", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class CourseController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Comment(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.course_lesson.CourseController/Comment",
            course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def FetchMasterCourses(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.course_lesson.CourseController/FetchMasterCourses",
            course__lesson_dot_grpc_dot_course__lesson__pb2.MasterRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.MasterResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def Like(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.course_lesson.CourseController/Like",
            course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def List(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.course_lesson.CourseController/List",
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseListRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def Rate(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.course_lesson.CourseController/Rate",
            course__lesson_dot_grpc_dot_course__lesson__pb2.RateRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.RateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def Retrieve(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.course_lesson.CourseController/Retrieve",
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def RetrieveComments(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.course_lesson.CourseController/RetrieveComments",
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveCommentsRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def RetrieveRates(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.course_lesson.CourseController/RetrieveRates",
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveRatesRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def UnLike(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.course_lesson.CourseController/UnLike",
            course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )


class GeneralControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel=None):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.channel = channel
        self.ready()

    def ready(self):
        if self.channel is not None:
            self.Retrieve = self.channel.unary_unary(
                "/config.course_lesson.GeneralController/Retrieve",
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.GeneralRetrieveRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.GeneralResponse.FromString,
            )


class GeneralControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Retrieve(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_GeneralControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "Retrieve": grpc.unary_unary_rpc_method_handler(
            servicer.Retrieve,
            request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.GeneralRetrieveRequest.FromString,
            response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.GeneralResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "config.course_lesson.GeneralController", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class GeneralController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Retrieve(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.course_lesson.GeneralController/Retrieve",
            course__lesson_dot_grpc_dot_course__lesson__pb2.GeneralRetrieveRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.GeneralResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )


class LessonControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel=None):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.channel = channel
        self.ready()

    def ready(self):
        if self.channel is not None:
            self.Comment = self.channel.unary_unary(
                "/config.course_lesson.LessonController/Comment",
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentResponse.FromString,
            )
            self.List = self.channel.unary_unary(
                "/config.course_lesson.LessonController/List",
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.LessonListRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.LessonResponse.FromString,
            )
            self.Retrieve = self.channel.unary_unary(
                "/config.course_lesson.LessonController/Retrieve",
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.LessonRetrieveRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.LessonResponse.FromString,
            )
            self.RetrieveComments = self.channel.unary_unary(
                "/config.course_lesson.LessonController/RetrieveComments",
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.LessonRetrieveCommentsRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.LessonResponse.FromString,
            )


class LessonControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Comment(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def List(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def Retrieve(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def RetrieveComments(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_LessonControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "Comment": grpc.unary_unary_rpc_method_handler(
            servicer.Comment,
            request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentRequest.FromString,
            response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentResponse.SerializeToString,
        ),
        "List": grpc.unary_unary_rpc_method_handler(
            servicer.List,
            request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.LessonListRequest.FromString,
            response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.LessonResponse.SerializeToString,
        ),
        "Retrieve": grpc.unary_unary_rpc_method_handler(
            servicer.Retrieve,
            request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.LessonRetrieveRequest.FromString,
            response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.LessonResponse.SerializeToString,
        ),
        "RetrieveComments": grpc.unary_unary_rpc_method_handler(
            servicer.RetrieveComments,
            request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.LessonRetrieveCommentsRequest.FromString,
            response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.LessonResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "config.course_lesson.LessonController", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class LessonController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Comment(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.course_lesson.LessonController/Comment",
            course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def List(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.course_lesson.LessonController/List",
            course__lesson_dot_grpc_dot_course__lesson__pb2.LessonListRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.LessonResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def Retrieve(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.course_lesson.LessonController/Retrieve",
            course__lesson_dot_grpc_dot_course__lesson__pb2.LessonRetrieveRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.LessonResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def RetrieveComments(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.course_lesson.LessonController/RetrieveComments",
            course__lesson_dot_grpc_dot_course__lesson__pb2.LessonRetrieveCommentsRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.LessonResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )


class MasterCourseControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel=None):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.channel = channel
        self.ready()

    def ready(self):
        if self.channel is not None:
            self.Comment = self.channel.unary_unary(
                "/config.course_lesson.MasterCourseController/Comment",
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentResponse.FromString,
            )
            self.FetchMasterCourses = self.channel.unary_unary(
                "/config.course_lesson.MasterCourseController/FetchMasterCourses",
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.MasterRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.MasterResponse.FromString,
            )
            self.Like = self.channel.unary_unary(
                "/config.course_lesson.MasterCourseController/Like",
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeResponse.FromString,
            )
            self.List = self.channel.unary_unary(
                "/config.course_lesson.MasterCourseController/List",
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseListRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
            )
            self.Rate = self.channel.unary_unary(
                "/config.course_lesson.MasterCourseController/Rate",
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.RateRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.RateResponse.FromString,
            )
            self.Retrieve = self.channel.unary_unary(
                "/config.course_lesson.MasterCourseController/Retrieve",
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
            )
            self.RetrieveComments = self.channel.unary_unary(
                "/config.course_lesson.MasterCourseController/RetrieveComments",
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveCommentsRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
            )
            self.RetrieveRates = self.channel.unary_unary(
                "/config.course_lesson.MasterCourseController/RetrieveRates",
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveRatesRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
            )
            self.UnLike = self.channel.unary_unary(
                "/config.course_lesson.MasterCourseController/UnLike",
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeResponse.FromString,
            )


class MasterCourseControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Comment(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def FetchMasterCourses(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def Like(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def List(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def Rate(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def Retrieve(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def RetrieveComments(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def RetrieveRates(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def UnLike(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_MasterCourseControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "Comment": grpc.unary_unary_rpc_method_handler(
            servicer.Comment,
            request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentRequest.FromString,
            response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentResponse.SerializeToString,
        ),
        "FetchMasterCourses": grpc.unary_unary_rpc_method_handler(
            servicer.FetchMasterCourses,
            request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.MasterRequest.FromString,
            response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.MasterResponse.SerializeToString,
        ),
        "Like": grpc.unary_unary_rpc_method_handler(
            servicer.Like,
            request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeRequest.FromString,
            response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeResponse.SerializeToString,
        ),
        "List": grpc.unary_unary_rpc_method_handler(
            servicer.List,
            request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseListRequest.FromString,
            response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.SerializeToString,
        ),
        "Rate": grpc.unary_unary_rpc_method_handler(
            servicer.Rate,
            request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.RateRequest.FromString,
            response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.RateResponse.SerializeToString,
        ),
        "Retrieve": grpc.unary_unary_rpc_method_handler(
            servicer.Retrieve,
            request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveRequest.FromString,
            response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.SerializeToString,
        ),
        "RetrieveComments": grpc.unary_unary_rpc_method_handler(
            servicer.RetrieveComments,
            request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveCommentsRequest.FromString,
            response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.SerializeToString,
        ),
        "RetrieveRates": grpc.unary_unary_rpc_method_handler(
            servicer.RetrieveRates,
            request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveRatesRequest.FromString,
            response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.SerializeToString,
        ),
        "UnLike": grpc.unary_unary_rpc_method_handler(
            servicer.UnLike,
            request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeRequest.FromString,
            response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "config.course_lesson.MasterCourseController", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class MasterCourseController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Comment(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.course_lesson.MasterCourseController/Comment",
            course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.AddCommentResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def FetchMasterCourses(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.course_lesson.MasterCourseController/FetchMasterCourses",
            course__lesson_dot_grpc_dot_course__lesson__pb2.MasterRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.MasterResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def Like(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.course_lesson.MasterCourseController/Like",
            course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def List(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.course_lesson.MasterCourseController/List",
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseListRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def Rate(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.course_lesson.MasterCourseController/Rate",
            course__lesson_dot_grpc_dot_course__lesson__pb2.RateRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.RateResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def Retrieve(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.course_lesson.MasterCourseController/Retrieve",
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def RetrieveComments(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.course_lesson.MasterCourseController/RetrieveComments",
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveCommentsRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def RetrieveRates(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.course_lesson.MasterCourseController/RetrieveRates",
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseRetrieveRatesRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.CourseResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def UnLike(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.course_lesson.MasterCourseController/UnLike",
            course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.TriggerLikeResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )


class PackageControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel=None):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.channel = channel
        self.ready()

    def ready(self):
        if self.channel is not None:
            self.List = self.channel.unary_unary(
                "/config.course_lesson.PackageController/List",
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.PackageListRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.PackageResponse.FromString,
            )
            self.Retrieve = self.channel.unary_unary(
                "/config.course_lesson.PackageController/Retrieve",
                request_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.PackageRetrieveRequest.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.PackageResponse.FromString,
            )


class PackageControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def List(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def Retrieve(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_PackageControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "List": grpc.unary_unary_rpc_method_handler(
            servicer.List,
            request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.PackageListRequest.FromString,
            response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.PackageResponse.SerializeToString,
        ),
        "Retrieve": grpc.unary_unary_rpc_method_handler(
            servicer.Retrieve,
            request_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.PackageRetrieveRequest.FromString,
            response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.PackageResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "config.course_lesson.PackageController", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class PackageController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def List(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.course_lesson.PackageController/List",
            course__lesson_dot_grpc_dot_course__lesson__pb2.PackageListRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.PackageResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def Retrieve(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.course_lesson.PackageController/Retrieve",
            course__lesson_dot_grpc_dot_course__lesson__pb2.PackageRetrieveRequest.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.PackageResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )


class TopBannerControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel=None):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.channel = channel
        self.ready()

    def ready(self):
        if self.channel is not None:
            self.RetrieveCurrent = self.channel.unary_unary(
                "/config.course_lesson.TopBannerController/RetrieveCurrent",
                request_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
                response_deserializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TopBannerResponse.FromString,
            )


class TopBannerControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def RetrieveCurrent(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_TopBannerControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "RetrieveCurrent": grpc.unary_unary_rpc_method_handler(
            servicer.RetrieveCurrent,
            request_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            response_serializer=course__lesson_dot_grpc_dot_course__lesson__pb2.TopBannerResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "config.course_lesson.TopBannerController", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class TopBannerController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def RetrieveCurrent(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/config.course_lesson.TopBannerController/RetrieveCurrent",
            google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            course__lesson_dot_grpc_dot_course__lesson__pb2.TopBannerResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )
